"""Pacman, classic arcade game.

Exercises

1. Change the board.
2. Change the number of monsters.
3. Change where pacman starts.
4. Make the monsters faster/slower.
5. Make the monsters smarter.

"""

from random import *
from turtle import *
from freegames import floor, vector


aim = vector(5, 0)
pacman = vector(-40, -80)
tiles = [
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
    0, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0,
    0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
    0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0,
    0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0,
    0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0,
    0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
    0, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0,
    0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0,
    0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
    0, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0,
    0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0,
    0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0,
    0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0,
    0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0,
    0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
]

def square(x, y):
    up()
    goto(x, y)
    down()
    begin_fill()
    for count in range(4):
        forward(20)
        left(90)
    end_fill()

def offset(point):
    x = (floor(point.x, 20) + 200) / 20
    y = (180 - floor(point.y, 20)) / 20
    index = int(x + y * 20)
    return index

def valid(man, aim):
    step = man + aim
    index = offset(step)

    if tiles[index] == 0:
        return False

    step.x += 19

    step.move(19)
    index = offset(step)

    return tiles[index] != 0

def move(man, aim):
    if valid(man, aim):
        man.move(aim)

def draw():
    clear()
    color('blue')

    for index in range(len(tiles)):
        tile = tiles[index]

        if tile > 0:
            x = (index % 20) * 20 - 200
            y = 180 - (index // 20) * 20
            square(x, y)

            if tile == 1:
                up()
                goto(x + 10, y + 10)
                dot(2, 'white')

    move(pacman, aim)
    up()
    goto(pacman.x + 10, pacman.y + 10)
    dot(20, 'yellow')

    for index in range(400):
        x = index % 20 * 20 - 200
        y = 180 - (index // 20) * 20
        goto(x, y)
        color('white')
        write(index)
    
    update()
    ontimer(draw, 50)

def change(x, y):
    aim.x = x
    aim.y = y

setup(420, 420, 370, 0)
bgcolor('black')
hideturtle()
tracer(False)
listen()
onkey(lambda: change(5, 0), 'Right')
onkey(lambda: change(-5, 0), 'Left')
onkey(lambda: change(0, 5), 'Up')
onkey(lambda: change(0, -5), 'Down')
draw()
done()

import sys; sys.exit(0)




import sys
import pygame
import random
from pygame.locals import *

pygame.init()

font = pygame.font.Font(None, 24)
pacman_img = pygame.image.load('pacman.jpg')
ghost_img = pygame.image.load('ghost.jpg')
clock = pygame.time.Clock()
screen = pygame.display.set_mode((340, 400))

class Game:
    """Track pacman game state."""
    def __init__(self):
        """Initialize game."""
        self.tiles_width = 17
        self.levels = [
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
             0, 2, 2, 2, 2, 2, 2, 2, 0, 2, 2, 2, 2, 2, 2, 2, 0,
             0, 2, 0, 0, 2, 0, 0, 2, 0, 2, 0, 0, 2, 0, 0, 2, 0,
             0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0,
             0, 2, 0, 0, 2, 0, 2, 0, 0, 0, 2, 0, 2, 0, 0, 2, 0,
             0, 2, 2, 2, 2, 0, 2, 2, 0, 2, 2, 0, 2, 2, 2, 2, 0,
             0, 2, 0, 0, 2, 0, 0, 2, 0, 2, 0, 0, 2, 0, 0, 0, 0,
             0, 2, 0, 0, 2, 0, 2, 2, 2, 2, 2, 0, 2, 0, 0, 0, 0,
             0, 2, 2, 2, 2, 2, 2, 0, 0, 0, 2, 2, 2, 2, 2, 2, 0,
             0, 0, 0, 0, 2, 0, 2, 2, 2, 2, 2, 0, 2, 0, 0, 2, 0,
             0, 0, 0, 0, 2, 0, 2, 0, 0, 0, 2, 0, 2, 0, 0, 2, 0,
             0, 2, 2, 2, 2, 2, 2, 2, 0, 2, 2, 2, 2, 2, 2, 2, 0,
             0, 2, 0, 0, 2, 0, 0, 2, 0, 2, 0, 0, 2, 0, 0, 2, 0,
             0, 2, 2, 0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 2, 2, 0,
             0, 0, 2, 0, 2, 0, 2, 0, 0, 0, 2, 0, 2, 0, 2, 0, 0,
             0, 2, 2, 2, 2, 0, 2, 2, 0, 2, 2, 0, 2, 2, 2, 2, 0,
             0, 2, 0, 0, 0, 0, 0, 2, 0, 2, 0, 0, 0, 0, 0, 2, 0,
             0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0,
             0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
        ]
        self.reset()

    def reset(self):
        """Reset game to initial game state."""
        self.running = True
        self.score = 0
        self.level = 0
        self.next_dir = 0
        self.pacman = [pygame.Rect(160, 260, 20, 20), 0]
        self.ghosts = [[pygame.Rect(20, 20, 20, 20), 0],
                       [pygame.Rect(300, 20, 20, 20), 2],
                       [pygame.Rect(20, 340, 20, 20), 0],
                       [pygame.Rect(300, 340, 20, 20), 2]]

    def next_pos(self, next_dir, rect):
        """Advance rectangle by given direction."""
        if next_dir == 0: chng = (4, 0)
        if next_dir == 1: chng = (0, -4)
        if next_dir == 2: chng = (-4, 0)
        if next_dir == 3: chng = (0, 4)
        return rect.move(*chng)

    def rect_idx(self, rect):
        """Calculate tile index by rectangle."""
        return (rect.top / 20) * self.tiles_width + (rect.left / 20)

    def valid_pos(self, rect):
        """Return True iff character rectangle is valid on tiles."""
        prev_tile = self.levels[self.level][self.rect_idx(rect)]
        next_tile = self.levels[self.level][self.rect_idx(rect.move(19, 19))]
        on_track = rect.left % 20 == 0 or rect.top % 20 == 0
        return prev_tile != 0 and next_tile != 0 and on_track

    def move_rect(self, next_dir, curr_dir, rect):
        """
        Move character rectangle.
        Returns next character location and direction.
        """
        next_rect = self.next_pos(next_dir, rect)
        if self.valid_pos(next_rect):
            return (next_rect, next_dir)
        else:
            next_rect = self.next_pos(curr_dir, rect)
            if self.valid_pos(next_rect):
                return (next_rect, curr_dir)
        return (rect, curr_dir)

    def move_pacman(self):
        """Move pacman character."""
        rect, curr_dir = self.pacman
        tup = self.move_rect(self.next_dir, curr_dir, rect)
        self.pacman[0], self.pacman[1] = tup

    def move_ghosts(self):
        """Move ghost characters."""
        def dist_pacman(rect):
            return ((rect.top - self.pacman[0].top) ** 2
                    + (rect.left - self.pacman[0].left) ** 2) ** 0.5

        def test_dir(new_dir, curr_dir, rect):
            next_tup = self.move_rect(new_dir, curr_dir, rect)
            return dist_pacman(next_tup[0])

        for tup in self.ghosts:
            rect = tup[0]
            curr_dir = next_dir = tup[1]
            if rect.top % 20 == 0 and rect.left % 20 == 0:

                # Uncomment for Ghost AI
                #
                # dist = [(test_dir(val, curr_dir, rect), val)
                #         for val in (0, 1, 2, 3)]
                # min_dist = min(dist)
                # next_dir = min_dist[1]

                next_dir = (curr_dir + random.choice((1, 3))) % 4

            tup[0], tup[1] = self.move_rect(next_dir, curr_dir, rect)

    def update(self):
        """Update tiles, score and running based on pacman location."""
        self.next_dir = self.pacman[1]

        idx = self.rect_idx(self.pacman[0])
        if self.levels[self.level][idx] == 2:
            self.score += 1
            self.levels[self.level][idx] = 1

        has_food = any(val == 2 for val in self.levels[self.level])
        ghost_rects = [tup[0] for tup in self.ghosts]
        is_eaten = self.pacman[0].collidelist(ghost_rects) >= 0
        self.running = has_food and not is_eaten

    def draw(self):
        """Draw game on screen."""
        # Draw tiles

        for idx, val in enumerate(self.levels[self.level]):
            left = (idx % self.tiles_width) * 20
            top = (idx / self.tiles_width) * 20
            if val == 0:
                # Draws a blue square.
                color = (0, 0, 128)
                pygame.draw.rect(screen, color, (left, top, 20, 20))
            else:
                # Draw a black square.
                color = (0, 0, 0)
                pygame.draw.rect(screen, color, (left, top, 20, 20))
            if val == 2:
                # Draw a white dot.
                color = (255, 255, 255)
                pygame.draw.circle(screen, color, (left + 10, top + 10), 2)

        # Draw pacman.

        pacman_dir = pygame.transform.rotate(pacman_img, 90 * self.pacman[1])
        screen.blit(pacman_dir, self.pacman[0])

        # Draw ghosts.

        for val in self.ghosts:
            screen.blit(ghost_img, val[0])

        # Draw score.

        pygame.draw.rect(screen, (0, 0, 0), (0, 380, 100, 20))
        msg = 'Score: ' + str(self.score)
        text = font.render(msg , True, (255, 255, 255))
        screen.blit(text, (5, 382))

game = Game()

while True:

    event = pygame.event.poll()

    # Respond to key presses.

    if event.type == pygame.QUIT:
        pygame.quit()
        sys.exit()
    elif event.type == KEYDOWN:
        if event.key == K_UP:
            game.next_dir = 1
        elif event.key == K_RIGHT:
            game.next_dir = 0
        elif event.key == K_DOWN:
            game.next_dir = 3
        elif event.key == K_LEFT:
            game.next_dir = 2
        elif event.key == K_r:
            game.reset()
        elif event.key == K_q:
            pygame.event.post(pygame.event.Event(QUIT))

    if game.running:
        game.move_pacman()
        game.move_ghosts()
        game.update()

    game.draw()

    # Display next frame.

    pygame.display.flip()
    clock.tick(12)



"""Cannon, hitting targets with projectiles.

Exercises

1. Keep score by counting target hits.
2. Vary the effect of gravity.
3. Apply gravity to the targets.
4. Change the speed of the ball.

"""

from random import randrange
from turtle import *
from freegames import vector

ball = vector(-200, -200)
speed = vector(0, 0)
targets = []

def tap(x, y):
    "Respond to screen tap."
    if not inside(ball):
        ball.x = -199
        ball.y = -199
        speed.x = (x + 200) / 25
        speed.y = (y + 200) / 25

def inside(xy):
    "Return True if xy within screen."
    return -200 < xy.x < 200 and -200 < xy.y < 200

def draw():
    "Draw ball and targets."
    clear()

    for target in targets:
        goto(target.x, target.y)
        dot(20, 'blue')

    if inside(ball):
        goto(ball.x, ball.y)
        dot(6, 'red')

    update()

def move():
    "Move ball and targets."
    if randrange(40) == 0:
        y = randrange(-150, 150)
        target = vector(200, y)
        targets.append(target)

    for target in targets:
        target.x -= 0.5

    if inside(ball):
        speed.y -= 0.35
        ball.move(speed)

    dupe = targets.copy()
    targets.clear()

    for target in dupe:
        if abs(target - ball) > 13:
            targets.append(target)

    draw()

    for target in targets:
        if not inside(target):
            return

    ontimer(move, 50)

setup(420, 420, 370, 0)
hideturtle()
up()
tracer(False)
onscreenclick(tap)
move()
done()



"""Connect Four

Exercises

1. Change the colors.
2. Draw squares instead of circles for open spaces.
3. Add logic to detect a full row.
4. Create a random computer player.
5. How would you detect a winner?

"""

from turtle import *

turns = {'red': 'yellow', 'yellow': 'red'}
state = {'player': 'yellow', 'rows': [0] * 8}

def line(a, b, x, y):
    "Draw line from (a, b) to (x, y)."
    up()
    goto(a, b)
    down()
    goto(x, y)

def grid():
    "Draw Connect Four grid."
    bgcolor('light blue')

    for x in range(-150, 200, 50):
        line(x, -200, x, 200)

    for x in range(-175, 200, 50):
        for y in range(-175, 200, 50):
            up()
            goto(x, y)
            dot(40, 'white')

    update()

def tap(x, y):
    "Draw red or yellow circle in tapped row."
    player = state['player']
    rows = state['rows']

    row = int((x + 200) // 50)
    count = rows[row]

    x = ((x + 200) // 50) * 50 - 200 + 25
    y = count * 50 - 200 + 25

    up()
    goto(x, y)
    dot(40, player)
    update()

    rows[row] = count + 1
    state['player'] = turns[player]

setup(420, 420, 370, 0)
hideturtle()
tracer(False)
grid()
onscreenclick(tap)
done()



"""Bagels, a number puzzle game.

Exercises:

1. Can you guess the number?
2. How much harder is 6 digits? Do you need more guesses?
3. What's the maximum number of digits we could support?

"""

from random import sample, shuffle

digits = 3
guesses = 10

print('I am thinking of a', digits, 'digit number.')
print('Try to guess what it is.')
print('Here are some clues:')
print('When I say:    That means:')
print('  pico         One digit is correct but in the wrong position.')
print('  fermi        One digit is correct and in the right position.')
print('  bagels       No digit is correct.')
print('There are no repeated digits in the number.')

# Create a random number.

letters = sample('0123456789', digits)

if letters[0] == '0':
    letters.reverse()

number = ''.join(letters)

print('I have thought up a number.')
print('You have', guesses, 'guesses to get it.')

counter = 1

while True:
    print('Guess #', counter)
    guess = input()

    if len(guess) != digits:
        print('Wrong number of digits. Try again!')
        continue

    # Create the clues.

    clues = []

    for index in range(len(guess)):
        if guess[index] == number[index]:
            clues.append('fermi')
        elif guess[index] in number:
            clues.append('pico')

    shuffle(clues)

    if len(clues) == 0:
        print('bagels')
    else:
        print(' '.join(clues))

    counter += 1

    if guess == number:
        print('You got it!')
        break

    if counter > guesses:
        print('You ran out of guesses. The answer was', number)
        break



"""Flappy, game inspired by Flappy Bird.

Exercises

1. Keep score.
2. Vary the speed.
3. Vary the size of the balls.
4. Allow the bird to move forward and back.

"""

from random import *
from turtle import *
from freegames import vector

bird = vector(0, 0)
balls = []

def tap(x, y):
    "Move bird up in response to screen tap."
    up = vector(0, 30)
    bird.move(up)

def inside(point):
    "Return True if point on screen."
    return -200 < point.x < 200 and -200 < point.y < 200

def draw():
    "Update objects and draw screen."
    bird.y -= 5

    for ball in balls:
        ball.x -= 3

    if randrange(10) == 0:
        y = randrange(-199, 199)
        ball = vector(199, y)
        balls.append(ball)
        
    alive = inside(bird)

    for ball in balls:
        dodge = abs(ball - bird) > 15
        alive = alive and dodge

    if len(balls) > 30:
        balls.pop(0)

    clear()

    for ball in balls:
        goto(ball.x, ball.y)
        dot(20, 'black')

    goto(bird.x, bird.y)

    if alive:
        dot(10, 'green')
    else:
        dot(10, 'red')

    update()

    if alive:
        ontimer(draw, 50)

setup(420, 420, 370, 0)
hideturtle()
up()
tracer(False)
onscreenclick(tap)
draw()
done()



"""Guess a number within a range.

Exercises

1. Change the range to be from 0 to 1,000,000.
2. Can you still guess the number?
3. Print the number of guesses made.
4. Limit the number of guesses to the minimum required.

"""

from random import randint

start = 0
end = 100
value = randint(start, end)

print("I'm thinking of a number between", start, "and", end)

guess = None

while guess != value:

    text = input("Guess the number: ")

    try:
        guess = int(text)
    except:
        print("Whoops! Be sure the number contains only digits.")
        continue

    if guess < value:
        print("Higher.")
    elif guess > value:
        print("Lower.")

print("Congratulations! You guessed the right answer:", value)



from .utils import floor, path, vector

__title__ = 'freegames'
__version__ = '1.4.3'
__build__ = 0x010403
__author__ = 'Grant Jenks'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2017 Grant Jenks'



"""Crypto: tool for encrypting and decrypting messages.

Exercises

1. Review 'ord' and 'chr' functions and letter-to-number mapping.
2. Explain what happens if you use key 26.
3. Find a way to decode a message without a key.
4. Encrypt numbers.
5. Make the encryption harder to decode.

"""

def encrypt(message, key):
    result = ''

    # Iterate letters in message and encrypt each individually.

    for letter in message:
        if letter.isalpha():

            # Letters are numbered like so:
            # A, B, C - Z is 65, 66, 67 - 90
            # a, b, c - z is 97, 98, 99 - 122

            num = ord(letter)

            if letter.isupper():
                base = ord('A')
            elif letter.islower():
                base = ord('a')

            # The encryption equation:

            num = (num - base + key) % 26 + base

            result += chr(num)

        elif letter.isdigit():

            # TODO: Encrypt digits.
            result += letter

        else:
            result += letter

    return result

def decrypt(message, key):
    return encrypt(message, -key)

def decode(message):
    # TODO: Decode a message without a key.
    pass

def get_key():
    try:
        key = int(input('Enter a key (1 - 25): '))
        return key
    except:
        print('Invalid key. Using key: 0.')
        return 0

if __name__ == '__main__':
    print('Do you wish to encrypt, decrypt, or decode a message?')
    choice = input()

    if choice == 'encrypt':
        message = input('Message: ')
        key = get_key()
        print('Encrypted message:', encrypt(message, key))

    elif choice == 'decrypt':
        message = input('Message: ')
        key = get_key()
        print('Decrypted message:', decrypt(message, key))

    elif choice == 'decode':
        message = input('Message: ')
        print('Decoding message:')
        decode(message)

    else:
        print('Error: Unrecognized Command')



"""Pong, classic arcade game.

Exercises

1. Change the colors.
2. What is the frame rate? Make it faster or slower.
3. Change the speed of the ball.
4. Change the size of the paddles.
5. Change how the ball bounces off walls.
6. How would you add a computer player?
6. Add a second ball.

"""

from random import choice, random
from turtle import *
from freegames import vector

def value():
    "Randomly generate value between (-5, -3) or (3, 5)."
    return (3 + random() * 2) * choice([1, -1])

ball = vector(0, 0)
aim = vector(value(), value())
state = {1: 0, 2: 0}

def move(player, change):
    "Move player position by change."
    state[player] += change

def rectangle(x, y, width, height):
    "Draw rectangle at (x, y) with given width and height."
    up()
    goto(x, y)
    down()
    begin_fill()
    for count in range(2):
        forward(width)
        left(90)
        forward(height)
        left(90)
    end_fill()

def draw():
    "Draw game and move pong ball."
    clear()
    rectangle(-200, state[1], 10, 50)
    rectangle(190, state[2], 10, 50)

    ball.move(aim)
    x = ball.x
    y = ball.y

    up()
    goto(x, y)
    dot(10)
    update()

    if y < -200 or y > 200:
        aim.y = -aim.y

    if x < -185:
        low = state[1]
        high = state[1] + 50

        if low <= y <= high:
            aim.x = -aim.x
        else:
            return

    if x > 185:
        low = state[2]
        high = state[2] + 50

        if low <= y <= high:
            aim.x = -aim.x
        else:
            return

    ontimer(draw, 50)


setup(420, 420, 370, 0)
hideturtle()
tracer(False)
listen()
onkey(lambda: move(1, 20), 'w')
onkey(lambda: move(1, -20), 's')
onkey(lambda: move(2, 20), 'i')
onkey(lambda: move(2, -20), 'k')
draw()
done()



"""Snake, classic arcade game.

Excercises

1. How do you make the snake faster or slower?
2. How can you make the snake go around the edges?
3. How would you move the food?
4. Change the snake to respond to arrow keys.

"""

from turtle import *
from random import randrange
from freegames import vector

food = vector(0, 0)
snake = [vector(10, 0)]
aim = vector(0, -10)

def square(x, y, name):
    "Draw square at x, y with color name."
    up()
    goto(x, y)
    down()
    color(name)
    begin_fill()
    for i in range(4):
        forward(9)
        left(90)
    end_fill()

def change(x, y):
    "Change snake direction."
    aim.x = x
    aim.y = y

def inside(head):
    "Return True if head inside boundaries."
    return -200 < head.x < 190 and -200 < head.y < 190

def move():
    "Move snake forward one segment."
    head = snake[-1].copy()
    head.move(aim)

    if not inside(head) or head in snake:
        square(head.x, head.y, 'red')
        update()
        return

    snake.append(head)

    if head == food:
        print('Snake:', len(snake))
        food.x = randrange(-15, 15) * 10
        food.y = randrange(-15, 15) * 10
    else:
        snake.pop(0)

    clear()

    for body in snake:
        square(body.x, body.y, 'black')

    square(food.x, food.y, 'green')
    update()
    ontimer(move, 100)

setup(420, 420, 370, 0)
hideturtle()
tracer(False)
listen()
onkey(lambda: change(10, 0), 'Right')
onkey(lambda: change(-10, 0), 'Left')
onkey(lambda: change(0, 10), 'Up')
onkey(lambda: change(0, -10), 'Down')
move()
done()



"""Tic Tac Toe

Exercises

1. Give the X and O a different color and width.
2. What happens when someone taps a taken spot?
3. How would you detect when someone has won?
4. How could you create a computer player?

"""

from turtle import *

def line(a, b, x, y):
    "Draw line from (a, b) to (x, y)."
    up()
    goto(a, b)
    down()
    goto(x, y)

def grid():
    "Draw tic-tac-toe grid."
    line(-67, 200, -67, -200)
    line(67, 200, 67, -200)
    line(-200, -67, 200, -67)
    line(-200, 67, 200, 67)

def drawx(x, y):
    "Draw X player."
    line(x, y, x + 133, y + 133)
    line(x, y + 133, x + 133, y)

def drawo(x, y):
    "Draw O player."
    up()
    goto(x + 67, y + 5)
    down()
    circle(62)

def floor(value):
    return ((value + 200) // 133) * 133 - 200

state = {'player': 0}
players = [drawx, drawo]

def tap(x, y):
    "Draw X or O in tapped square."
    x = floor(x)
    y = floor(y)
    player = state['player']
    draw = players[player]
    draw(x, y)
    update()
    state['player'] = not player

setup(420, 420, 370, 0)
hideturtle()
tracer(False)
grid()
update()
onscreenclick(tap)
done()



"""Simon Says

Exercises

1. Speed up tile flash rate.
2. Add more tiles.

"""

from random import choice
from time import sleep
from turtle import *
from freegames import floor, vector

pattern = []
guesses = []
tiles = {
    vector(0, 0): ('red', 'dark red'),
    vector(0, -200): ('blue', 'dark blue'),
    vector(-200, 0): ('green', 'dark green'),
    vector(-200, -200): ('yellow', 'khaki'),
}

def square(x, y, name):
    "Draw square at (x, y) with color name."
    up()
    goto(x, y)
    down()
    color(name)
    begin_fill()
    for count in range(4):
        forward(200)
        left(90)
    end_fill()
    update()

def grid():
    "Draw grid of tiles."
    square(0, 0, 'dark red')
    square(0, -200, 'dark blue')
    square(-200, 0, 'dark green')
    square(-200, -200, 'khaki')

def flash(tile):
    "Flash tile in grid."
    glow, dark = tiles[tile]
    square(tile.x, tile.y, glow)
    sleep(0.5)
    square(tile.x, tile.y, dark)
    sleep(0.5)

def grow():
    "Grow pattern and flash tiles."
    tile = choice(list(tiles))
    pattern.append(tile)

    for tile in pattern:
        flash(tile)

    print('Pattern length:', len(pattern))
    guesses.clear()

def tap(x, y):
    "Respond to screen tap."
    onscreenclick(None)
    x = floor(x, 200)
    y = floor(y, 200)
    tile = vector(x, y)
    index = len(guesses)

    if tile != pattern[index]:
        exit()

    guesses.append(tile)
    flash(tile)

    if len(guesses) == len(pattern):
        grow()

    onscreenclick(tap)

def start(x, y):
    "Start game."
    grow()
    onscreenclick(tap)

setup(420, 420, 370, 0)
hideturtle()
tracer(False)
grid()
onscreenclick(start)
done()



import argparse
import os
import runpy

directory = os.path.dirname(os.path.realpath(__file__))
contents = os.listdir(directory)

def game_file(name):
    "Return True if filename represents a game."
    return (
        name.endswith('.py')
        and not name.startswith('__')
        and name != 'utils.py'
    )

games = sorted(name[:-3] for name in contents if game_file(name))

parser = argparse.ArgumentParser(
    prog='freegames',
    description='Free Python Games',
    epilog='Copyright 2017 Grant Jenks',
)
subparsers = parser.add_subparsers(dest='command', help='sub-command help')

parser_list = subparsers.add_parser('list', help='list games')

parser_copy = subparsers.add_parser('copy', help='copy game source code')
parser_copy.add_argument('game', choices=games, help='game name')
parser_copy.add_argument(
    '--force',
    action='store_true',
    help='overwrite existing file',
)

parser_show = subparsers.add_parser('show', help='show game source code')
parser_show.add_argument('game', choices=games, help='game name')

args = parser.parse_args()

if args.command == 'list':
    for game in games:
        print(game)
elif args.command == 'copy':
    filename = args.game + '.py'

    with open(os.path.join(directory, filename)) as reader:
        text = reader.read()

    cwd = os.getcwd()
    path = os.path.join(cwd, filename)

    if args.force or not os.path.exists(path):
        with open(path, 'w') as writer:
            writer.write(text)
    else:
        print('ERROR: File already exists. Specify --force to overwrite.')
else:
    assert args.command == 'show'
    
    with open(os.path.join(directory, args.game + '.py')) as reader:
        print(reader.read())
